/***************************************************************************
 *   Copyright (C) 2003 by jalal (Tim Pushman)                             *
 *   jalal@gnomedia.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "krefty_part.h"

#include <kinstance.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kfiledialog.h>

#include <qfile.h>
#include <qtextstream.h>
#include <qmultilineedit.h>

#include <khtmlview.h>
#include <khtml_part.h>

#include <dom/css_stylesheet.h>
#include <kparts/event.h>
#include <kdebug.h>

#include <kprocess.h>
#include <kmessagebox.h>
#include <klocale.h>

const QString sXSLFilename = "krefty.xsl"; // not really needed, but a bit of a safety net
const QString sXSLTPROC    = "xsltproc";

KreftyPart::KreftyPart( QWidget *parentWidget, const char */*widgetName*/,
                                  QObject *parent, const char *name, const QStringList &args  )
    : KParts::ReadOnlyPart(parent, name),m_parentWidget( parentWidget ),m_args(args),m_xslFile( sXSLFilename )
{
    // we need an instance
    setInstance( KreftyPartFactory::instance() );

    // setup the central display widget the way we like it
    m_pHTMLPart = new KHTMLPart( parentWidget );
    m_pHTMLPart->setJScriptEnabled(false);
    m_pHTMLPart->setJavaEnabled(false);
    m_pHTMLPart->setMetaRefreshEnabled(false);
    m_pHTMLPart->setPluginsEnabled(false);
    m_pHTMLPart->view()->setHScrollBarMode( QScrollView::AlwaysOff );
    m_pHTMLPart->view()->setFrameStyle( QFrame::Panel | QFrame::Sunken );
    m_pHTMLPart->view()->setFixedWidth( HTML_VIEW_WIDTH );
    //m_pHTMLPart->view()->setFixedHeight( 400 );

    // notify the part that this is our internal widget
    setWidget(m_pHTMLPart->view());

    // create our actions
    connect( m_pHTMLPart, SIGNAL( completed() ), this, SLOT( sheetLoaded() ) );

    // process to run 'xsltproc' in.
    m_pProc = new KProcess();
    Q_CHECK_PTR( m_pProc );

    connect( m_pProc, SIGNAL( processExited( KProcess* ) ), this, SLOT( transformDone( KProcess* ) ) );
    connect( m_pProc, SIGNAL( receivedStdout(KProcess*, char*, int) ), this, SLOT( htmlChunk(KProcess*, char*, int) ) );
    connect( m_pProc, SIGNAL( receivedStderr(KProcess*, char*, int) ), this, SLOT( transformError(KProcess*, char*, int) ) );

    QStringList::Iterator it = m_args.begin();
    for( ; it != m_args.end(); ++it ){
        int pos = (*it).find("=");
        if( pos > 0 ){
            QString key = (*it).left(pos);
            QString val = (*it).mid(pos+1);
            if( key == "xslFile" ){
                setXSLFile( val );
            }
        }
    }
}

KreftyPart::~KreftyPart()
{
    delete m_pProc;
}

// slot called when the HTML Part has finished loading.
// can't think of a use for it at the mo' ;)
void KreftyPart::sheetLoaded()
{
}

void KreftyPart::transformError(KProcess*, char* msg, int /*msglen*/ )
{
    kdDebug() << "*** ERROR: KreftyPart::transformError: " << msg << endl;
    KMessageBox::error( m_pHTMLPart->view(), msg );
}

void KreftyPart::transformDone( KProcess* )
{
    if( m_pProc->exitStatus() != 0 ){
        kdDebug() << "*** KreftyPart::transformDone: error, process returned: " << m_pProc->exitStatus() << endl;
    }
    m_pProc->closeStdout();

    m_pHTMLPart->end();
}

void KreftyPart::htmlChunk(KProcess* , char* buff, int bufflen)
{
    m_pHTMLPart->write( buff, bufflen );
}

/*
 returns from xsltproc (from the manpage):
       0: normal
       1: no argument
       2: too many parameters
       3: unknown option
       4: failed to parse the stylesheet
       5: error in the stylesheet
       6: error in one of the documents
       7: unsupported xsl:output method
       8: string parameter contains both quote and double-quotes
       9: internal processing error
       10: processing was stopped by a terminating message
*/

bool KreftyPart::openFile()
{
    // m_file and m_url are set by the openURL() from the parent.
    // officially, I'm supposed to use m_file, not m_url...

    m_pHTMLPart->begin( m_url ); // begin clears the page as well.
    m_pProc->clearArguments();
    QString xslFile = m_url.directory(false);
    xslFile += m_xslFile; //sXSLFilename;
    (*m_pProc) << sXSLTPROC << xslFile << m_file;

    kdDebug() << "KreftyPart::openURL: transforming " << m_file << " with " << xslFile << endl;
    if( !m_pProc->start( KProcess::NotifyOnExit, KProcess::AllOutput ) ) {
        // TODO: improve the error message/handling.
        kdDebug() << "*** ERROR: KreftyPart::openFile: Could not start process." << endl;
        KMessageBox::error( m_parentWidget, i18n( "Error running XSLT processor program: " ) );
        return false;
    }
    return true;
}

// It's usually safe to leave the factory code alone.. with the
// notable exception of the KAboutData data
#include <kaboutdata.h>
#include <klocale.h>

KInstance*  KreftyPartFactory::s_instance = 0L;
KAboutData* KreftyPartFactory::s_about = 0L;

KreftyPartFactory::KreftyPartFactory()
    : KParts::Factory()
{
}

KreftyPartFactory::~KreftyPartFactory()
{
    delete s_instance;
    delete s_about;

    s_instance = 0L;
}

KParts::Part* KreftyPartFactory::createPartObject( QWidget *parentWidget, const char *widgetName,
                                                        QObject *parent, const char *name,
                                                        const char */*classname*/, const QStringList &args )
{
    // Create an instance of our Part
    KreftyPart* obj = new KreftyPart( parentWidget, widgetName, parent, name, args );

    // See if we are to be read-write or not
    //if (QCString(classname) == "KParts::ReadOnlyPart")
    //    obj->setReadWrite(false);

    return obj;
}

KInstance* KreftyPartFactory::instance()
{
    if( !s_instance )
    {
        s_about = new KAboutData("kreftypart", I18N_NOOP("kreftyPart"), "0.2");
        s_about->addAuthor("jalal", 0, "jalal@gnomedia.com");
        s_instance = new KInstance(s_about);
    }
    return s_instance;
}

extern "C"
{
    void* init_libkreftypart()
    {
        return new KreftyPartFactory;
    }
};

#include "krefty_part.moc"
